#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.
import json
import time
from typing import Any, Dict, Optional, List
from openjiuwen.core.common.logging import logger

# MySQL LONGTEXT 最大长度约为 4GB，但为了防止内存问题，设置一个合理的上限
# 这里设置为 10MB，应该足够存储大多数正常输出，同时避免数据库问题
MAX_OUTPUT_LENGTH = 10 * 1024 * 1024


def _truncate_data(data: Any, max_length: int = MAX_OUTPUT_LENGTH) -> str:
    """
    将数据转换为字符串并截断到指定长度
    
    Args:
        data: 任意数据
        max_length: 最大长度限制
        
    Returns:
        截断后的字符串
    """
    if data is None:
        return ""

    try:
        if isinstance(data, str):
            text = data
        else:
            text = json.dumps(data, ensure_ascii=False, default=str)
    except (TypeError, ValueError):
        text = str(data)

    if len(text) > max_length:
        truncation_warning = f"\n...[Content truncated: exceeded {max_length} characters]"
        text = text[:max_length - len(truncation_warning)] + truncation_warning

    return text
from openjiuwen.core.session.stream import TraceSchema, OutputSchema
from openjiuwen.core.session.tracer.span import TraceAgentSpan, TraceWorkflowSpan
from openjiuwen.core.session.interaction.interaction import InteractionOutput
from openjiuwen_studio.core.manager.repositories.workflow_execution_repository import workflow_execution_repository
from openjiuwen_studio.core.manager.repositories.agent_execution_repository import agent_execution_repository
from openjiuwen_studio.core.manager.repositories.trace_detail_repository import trace_detail_repository

from openjiuwen_studio.core.common.message import (
    TraceResponse, ExecuteStatus, ExecuteResponseType,
    InteractionResponse, ExecuteResponse
)
from openjiuwen_studio.schemas.agent import AgentId
from openjiuwen_studio.schemas.workflow import WorkflowId
from openjiuwen_studio.schemas.trace_detail import TraceDetail

# Define constant locally to avoid circular import
EMPTY_NODE_ID_PREFIX = 'empty_node_'


def _workflowspan_2_tracedetail(business_type: str, workflowspan: TraceWorkflowSpan, mapping: Optional[Dict[str, str]] = None) -> TraceDetail:
    # 使用mapping获取span_name，如果不存在则使用原component_name
    span_name = workflowspan.component_name
    if mapping and workflowspan.invoke_id in mapping:
        span_name = mapping[workflowspan.invoke_id]

    return TraceDetail(
        space_id="",
        business_id="",  # 填入agent/workflow名字
        business_type=business_type,
        trace_id=workflowspan.trace_id,
        span_id=workflowspan.invoke_id,
        span_type=workflowspan.component_type,
        span_name=span_name,
        parent_span_id=workflowspan.parent_invoke_id,
        method="",
        psm="",
        logid="",
        platform_type="studio_workflow" if business_type == "WORKFLOW" else "studio_agent_workflow",
        start_time_micros=int(time.mktime(workflowspan.start_time.timetuple()) * 1e6 +
                              workflowspan.start_time.microsecond) if workflowspan.start_time else None,
        end_time_micros=int(time.mktime(workflowspan.end_time.timetuple()) * 1e6 +
                            workflowspan.end_time.microsecond) if workflowspan.end_time else None,
        status_code=workflowspan.status,
        input=_truncate_data(workflowspan.inputs),
        output=_truncate_data(workflowspan.outputs),
        attributes=None
    )



def _agentspan_2_tracedetail(business_type: str, agentspan: TraceAgentSpan, mapping: Optional[Dict[str, str]] = None) -> TraceDetail:
    # 使用mapping获取span_name，如果不存在则使用原name
    span_name = agentspan.name
    if mapping and agentspan.invoke_type in mapping:
        span_name = mapping[agentspan.invoke_type]

    return TraceDetail(
        space_id="",
        business_id="",  # 填入agent/workflow名字
        business_type=business_type,
        trace_id=agentspan.trace_id,
        span_id=agentspan.invoke_id,
        span_type=agentspan.invoke_type,
        span_name=span_name,
        parent_span_id=agentspan.parent_invoke_id,
        method="",
        psm="",
        logid="",
        platform_type="studio_agent",
        start_time_micros=int(time.mktime(agentspan.start_time.timetuple()) * 1e6 +
                              agentspan.start_time.microsecond) if agentspan.start_time else None,
        end_time_micros=int(time.mktime(agentspan.end_time.timetuple()) * 1e6 +
                            agentspan.end_time.microsecond) if agentspan.end_time else None,
        status_code="error" if agentspan.error else ("finish" if agentspan.end_time else "start"),
        input=_truncate_data(agentspan.inputs),
        output=_truncate_data(agentspan.outputs),
        attributes=None
    )


def get_trace_workflow_output(data):
    """
        获取trace_workflow中streamOutputs或outputs
    """
    outputs_value = None
    if data.outputs:
        outputs_value = data.outputs
    elif data.stream_outputs:
        # 从 stream_outputs 中提取并拼接所有文本值
        text_parts = []
        output_key = "output"

        for item in data.stream_outputs:
            # 第一种格式: {'output': 'value'}
            if isinstance(item, dict) and 'output' in item:
                text_parts.append(str(item['output']))

            # 第二种格式: {'type': 'end node stream', 'index': 0, 'payload': {'response': 'value'}}
            # 第三种格式: {'type': 'output', 'index': 0, 'payload': {'output': '111', 'result_type': 'answer'}}
            elif isinstance(item, dict) and 'payload' in item:
                payload = item.get('payload', {})
                if isinstance(payload, dict):
                    # 检查 payload 中是否有 response 字段
                    if 'response' in payload:
                        text_parts.append(str(payload['response']))
                        output_key = "response"
                    if 'output' in payload:
                        text_parts.append(str(payload['output']))
                        output_key = "response"

        if text_parts:
            combined_text = ''.join(text_parts)
            outputs_value = {output_key: combined_text}

    return outputs_value


def get_trace_workflow_input(data, output_value=None):
    """
    获取trace_workflow中streamInputs或inputs
    """
    inputs_value = {}

    # 首先处理inputs
    if data.inputs:
        inputs_value.update(data.inputs)

    # 处理stream_inputs，按不同的输出键合并
    if hasattr(data, 'stream_inputs') and data.stream_inputs:
        # 用于存储每个output键对应的值
        stream_values = {}

        for item in data.stream_inputs:
            if isinstance(item, dict):
                # 提取键值对，item可能是 {'output2': '从前'} 或类似格式
                for key, value in item.items():
                    if key not in stream_values:
                        stream_values[key] = []
                    stream_values[key].append(str(value))

        # 将每个output键的值合并成字符串
        for key, value_list in stream_values.items():
            combined_text = ''.join(value_list)
            inputs_value[key] = combined_text

    # 特殊处理：对于 UserInputComponent，如果 inputs 为空，则从 outputs 中提取
    if not inputs_value and output_value and data.component_type == "UserInputComponent":
        inputs_value = output_value.copy() if isinstance(output_value, dict) else {}

    if not inputs_value:
        inputs_value=None

    return inputs_value


def map_trace_execute_status(status: Optional[str]) -> ExecuteStatus:
    """将底层 trace status 映射为前端可识别执行状态。"""
    if status == "start":
        return ExecuteStatus.Start
    if status in "interrupted":
        return ExecuteStatus.Interrupted
    if status == "agent":
        return ExecuteStatus.Agent
    return ExecuteStatus.Finish


def result_convert(chunk: Any, business_type: str, mapping: Optional[Dict[str, str]] = None) -> tuple[None, None] | tuple[Any, TraceWorkflowSpan, TraceDetail] | \
                                                      tuple[Any, TraceAgentSpan, TraceDetail] | tuple[Any, None, None] | \
                                                      tuple[Any, None]:
    """
    将底层执行器返回的 chunk 转换为对外 API 响应格式。
    返回 (response_dict, trace_data_or_none)
    """
    # Workflow Trace
    if isinstance(chunk, TraceSchema) and chunk.type == "tracer_workflow":
        logger.debug(f"get chunk.type == tracer_workflow, chunk: {chunk}")
        data = TraceWorkflowSpan.model_validate(chunk.payload)
        if EMPTY_NODE_ID_PREFIX in data.invoke_id:
            return None, None, None

        output_value = get_trace_workflow_output(data)
        input_value = get_trace_workflow_input(data, output_value)
        data.inputs = input_value
        data.outputs = output_value

        # 使用component_name_map映射组件名称
        component_name = data.component_name
        if mapping and data.invoke_id in mapping:
            component_name = mapping[data.invoke_id]

        # 将映射后的组件名称设置回data中，确保trace log使用正确的名称
        data.component_name = component_name

        return ExecuteResponse(
            type=ExecuteResponseType.Trace,
            payload=TraceResponse(
                id=data.invoke_id,
                name=component_name,
                version="",
                description="",

                status=map_trace_execute_status(data.status),

                inputs=input_value,
                outputs=output_value,
                output_text="",
                error=data.error,

                start_time=data.start_time,
                end_time=data.end_time,

                parent_id=data.loop_node_id if data.loop_node_id else data.parent_invoke_id,
                loop_index=data.loop_index,
            ).model_dump()
        ).model_dump(), data, _workflowspan_2_tracedetail(business_type, data, mapping)

    # Agent Trace
    if isinstance(chunk, TraceSchema) and chunk.type == "tracer_agent":
        data = TraceAgentSpan.model_validate(chunk.payload)

        # 处理流式输出，合并outputs列表中的多个条目
        if hasattr(data, 'outputs') and data.outputs is not None:
            # 检查outputs是否为字典且包含outputs列表
            if isinstance(data.outputs, dict) and 'outputs' in data.outputs:

                outputs_list = data.outputs['outputs']
                # 如果outputs列表有多个条目，进行合并
                if outputs_list and isinstance(outputs_list, list) and len(outputs_list) > 1:
                    # 收集所有content内容
                    merged_content = ""
                    for item in outputs_list:
                        if isinstance(item, dict) and 'content' in item:
                            merged_content += str(item['content'])
                    # 创建合并后的条目，保留第一个条目的其他字段
                    if merged_content and outputs_list:
                        # 复制第一个条目并更新content
                        merged_item = outputs_list[0].copy()
                        merged_item['content'] = merged_content
                        # 替换outputs列表为仅包含合并条目的列表
                        data.outputs['outputs'] = [merged_item]

        return ExecuteResponse(
            type=ExecuteResponseType.Trace,
            payload=TraceResponse(
                id=data.invoke_id,
                name="",
                version="",
                description="",

                status=ExecuteStatus.Agent,

                inputs=data.inputs,
                outputs=data.outputs,
                output_text="",
                error=data.error,

                start_time=data.start_time,
                end_time=data.end_time,

                parent_id=data.parent_invoke_id,
                loop_index=None,
            ).model_dump()
        ).model_dump(), data, _agentspan_2_tracedetail(business_type, data, mapping)

    # Final Answer
    if isinstance(chunk, OutputSchema) and chunk.type == "answer":
        return None, None, None

    # Streaming output
    if isinstance(chunk, OutputSchema) and chunk.type == "output":
        if mapping:
            logger.debug(f"get node name and node id mapping: {mapping}")
            for nid in mapping:
                if chunk.payload.get("node_id") == nid:
                    chunk.payload["node_name"] = mapping.get(nid)
        logger.debug(f"chunk.type == output return chunk.payload: {chunk.payload}")
        return ExecuteResponse(type=ExecuteResponseType.Workflow, payload=chunk.payload).model_dump(), None, None

    # Streaming output
    if isinstance(chunk, OutputSchema) and chunk.type == "end node stream":
        # 将原有payload={"response": "你好"} 转换成 payload={"output": "你好", "result_type": "answer"}格式
        if isinstance(chunk.payload, dict) and "response" in chunk.payload:
            answer_value = chunk.payload.get("response", "")
            # 确保answer是字符串类型，如果不是则序列化为JSON字符串
            if not isinstance(answer_value, str):
                try:
                    answer_value = json.dumps(answer_value, ensure_ascii=False)
                except (TypeError, ValueError):
                    answer_value = str(answer_value)
            # 仅对最终答案做后处理
            if business_type == "AGENT" and answer_value and answer_value.lstrip().startswith("#"):
                answer_value = "\n\n" + answer_value

            transformed_payload = {
                "output": answer_value,
                "result_type": "answer",
                "node_id": "end_0",
                "node_name": "结束",
                "index": chunk.index
            }
        elif isinstance(chunk.payload, dict) and "output" in chunk.payload:
            output_value = chunk.payload.get("output", {})
            if isinstance(output_value, dict):
                # 使用list()将values转换为列表，然后取第一个
                if output_value:  # 确保字典不为空
                    output_value = next(iter(output_value.values()))
                else:
                    output_value = ""
            else:
                output_value = json.dumps(output_value, ensure_ascii=False)

            transformed_payload = {
                "output": output_value,
                "result_type": "answer",
                "node_id": "end_0",
                "node_name": "结束",
                "index": chunk.index
            }
        else:
            transformed_payload = chunk.payload
            if isinstance(transformed_payload, dict):
                transformed_payload["index"] = chunk.index

        response_type = ExecuteResponseType.Workflow
        if business_type == "AGENT":
            response_type = ExecuteResponseType.Agent
        # 仅在被改为 Agent 模式下为最终答案添加前缀，解决工作流和agent输出间衔接问题
        # 由于可能存在多个 end node stream（如循环输出），不能仅靠 index==0 判断
        is_agent_first_chunk = (business_type == "AGENT" and chunk.index == 0)
        is_valid_dict = (
                isinstance(transformed_payload, dict) and "output" in transformed_payload
        )
        if is_agent_first_chunk and is_valid_dict:
            output = transformed_payload["output"]
            if not isinstance(output, str):  # 可选安全检查
                output = str(output)
            if not output.startswith("\n\n"):
                transformed_payload["output"] = "\n\n" + output
        return ExecuteResponse(type=response_type, payload=transformed_payload).model_dump(), None, None

    # Agent workflow run result
    # not processing workflow_final type
    if isinstance(chunk, OutputSchema) and chunk.type == "workflow_final":
        if business_type == "AGENT":
            answer_value = None
            # 处理 output 情况
            if chunk.payload.get("output"):
                output_data = chunk.payload.get("output")
                # 如果 output 是字典，转换为 key: value 格式的字符串
                if isinstance(output_data, dict):
                    formatted_items = []
                    for key, value in output_data.items():
                        if not isinstance(value, str):
                            value = str(value)
                        formatted_items.append(f"{key}: {value}")
                    answer_value = "\n".join(formatted_items)
                else:
                    answer_value = str(output_data)
            # 处理 response 情况
            elif chunk.payload.get("response"):
                response_data = chunk.payload.get("response")
                if isinstance(response_data, dict):
                    formatted_items = []
                    for key, value in response_data.items():
                        if not isinstance(value, str):
                            value = str(value)
                        formatted_items.append(f"{key}: {value}")
                    answer_value = "\n".join(formatted_items)
                else:
                    answer_value = str(response_data)
            # 仅对最终答案做后处理
            if answer_value and answer_value.lstrip().startswith("#"):
                answer_value = "\n\n" + answer_value

            if answer_value:
                agent_payload = {"output": answer_value, "result_type": "answer"}
                return ExecuteResponse(type=ExecuteResponseType.Agent, payload=agent_payload).model_dump(), None, None
        return None, None, None

    # Agent Token 粒度输出处理
    if isinstance(chunk, OutputSchema) and chunk.type == "llm_output":
        # 仅在一二级标题等特殊字符时添加换行，虽然会部分增大行间距，但可以解决工作流和agent输出间衔接问题
        sp_title = ["#", "##"]
        is_agent_first_chunk = (business_type == "AGENT")
        is_sp_key = (
                isinstance(chunk.payload, dict) and chunk.payload.get("output") in sp_title
        )
        if is_agent_first_chunk and is_sp_key:
            chunk.payload["output"] = "\n\n" + chunk.payload.get("output")
        return ExecuteResponse(type=ExecuteResponseType.Agent, payload=chunk.payload).model_dump(), None, None

    # Interaction
    if (
            isinstance(chunk, OutputSchema)
            and chunk.type == "__interaction__"
            and isinstance(chunk.payload, InteractionOutput)
    ):
        payload = InteractionResponse(
            interaction_node=chunk.payload.id,
            interaction_msg=chunk.payload.value,
        )
        return ExecuteResponse(type=ExecuteResponseType.Interaction,
                               payload=payload.model_dump()).model_dump(), None, None

    logger.debug(f"not convert chunk: {chunk}")
    return None, None, None


async def save_trace_details(index: WorkflowId | AgentId, tracedetails: List[TraceDetail]) -> None:
    """
    批量保存trace details，优化性能
    
    Args:
        index: WorkflowId或AgentId
        tracedetails: TraceDetail列表
    """
    if not tracedetails:
        return

    # 设置space_id和business_id
    for tracedetail in tracedetails:
        tracedetail.space_id = index.space_id
        if tracedetail.business_type == "WORKFLOW":
            tracedetail.business_id = index.workflow_id
        else:
            tracedetail.business_id = index.agent_id

    # 使用批量保存
    trace_detail_repository.create_trace_details(tracedetails)


# 保存执行日志
async def save_execution_traces(
        index: WorkflowId | AgentId,
        trace_logs: list[TraceWorkflowSpan | TraceAgentSpan],
) -> None:
    """
    统一保存一次执行的所有 trace
    """
    try:
        if isinstance(index, WorkflowId):
            # 目前workflow_execution_repository使用trace_id作为索引，后续看时候要改成conversation_id
            workflow_execution_repository.create_workflow_execution_log(index, trace_logs)
        elif isinstance(index, AgentId):
            agent_execution_repository.create_agent_execution_log(index, trace_logs)
    except Exception as e:
        logger.error(f"Failed to save execution traces for {index}: {e}", exc_info=True)


# 处理错误
async def handle_stream_error(
        trace_logs: list[TraceWorkflowSpan | TraceAgentSpan],
        trace_details: list[TraceDetail],
        last_chunk: Any,
        error_code: int,
        error_msg: str,
        index: WorkflowId | AgentId
) -> Optional[str]:
    business_type = "WORKFLOW" if isinstance(index, WorkflowId) else "AGENT"
    trace_id: Optional[str] = None
    if last_chunk is not None and hasattr(last_chunk, "payload"):
        _, trace_data, trace_detail = result_convert(last_chunk, business_type)
        if trace_details and trace_detail is not None:
            if error_code == -2:
                trace_detail.output = f"Agent execution interrupted: {error_msg}"
                trace_detail.status_code = 'interrupted'
            else:
                trace_detail.output = f"Agent execution error: {error_msg}"
                trace_detail.status_code = 'error'
            trace_details.append(trace_detail)
            trace_id = trace_detail.trace_id
        if trace_logs and trace_data is not None:
            if hasattr(trace_data, 'error') and hasattr(trace_data, 'status'):
                trace_data.status = "error"
                trace_data.error = {str(error_code): error_msg}
                trace_logs.append(trace_data)

    # 批量保存错误追踪详情
    if trace_details:
        try:
            await save_trace_details(index, trace_details)
        except Exception as e:
            logger.error(f"Failed to save trace details batch: {e}")

    if not trace_logs:
        logger.error(f"Workflow/Agent failed before any trace was emitted: {error_msg}")
    else:
        await save_execution_traces(index, trace_logs)
    return trace_id



import logging
from typing import Optional, Dict, Any, List

from sqlalchemy.orm import Session

from openjiuwen_studio.core.exceptions import ModelConfigNameExistsError, ValidationError, ModelConfigNotFoundError
from openjiuwen_studio.core.manager.model_manager.utils import SecurityUtils
from openjiuwen_studio.core.manager.repositories import EmbeddingModelConfigRepository, \
    JiuwenBaseRepository
from openjiuwen_studio.core.manager.repositories.system_embedding_model_repository import SystemEmbeddingModelRepository
from openjiuwen_studio.models import SpaceDB, SystemEmbeddingModelDB
from openjiuwen_studio.schemas.system_embedding_model import SystemEmbeddingModelCreate, SystemEmbeddingModelUpdate

logger = logging.getLogger(__name__)


class SystemEmbeddingModelManager:
    """Manages system embedding model operations including CRUD and validation."""

    def __init__(self, db: Session):
        """Initialize system model manager.

        Args:
            db: Database session
        """
        self.db = db
        self.system_model_repo = SystemEmbeddingModelRepository(db)
        self.model_repo = EmbeddingModelConfigRepository(db)
        self.space_repo = JiuwenBaseRepository(db, SpaceDB)
        self.security_utils = SecurityUtils()

    def create_system_embedding_model(self, model: SystemEmbeddingModelCreate) -> SystemEmbeddingModelDB:
        """Add a system embedding model to the database.
        Args:
            model: SystemEmbeddingModelCreate
        Returns:
            Created system embedding model obj
        """
        try:
            # check if model exists
            if self.system_model_repo.check_model_exists(name=model.model_name):
                raise ModelConfigNameExistsError(f"System embedding model name already exists: {model.model_name}")

            model_dict = model.model_dump(exclude_unset=True, exclude_none=True)

            # Encrypt API key
            encrypted_api_key = self.security_utils.encrypt_api_key(model.api_key) if model.api_key else None
            if encrypted_api_key:
                model_dict.update({'api_key': encrypted_api_key})

            # add system model
            system_model = self.system_model_repo.create(model_dict)
            logger.info(f"Created system embedding model name: {system_model.model_name}, id: {system_model.id}")

            # add this model for all users
            model_dict.update({'is_system_model': True})
            model_dict.update({'system_model_id': system_model.id})

            all_spaces = self.space_repo.query().all()
            for space in all_spaces:
                if self.model_repo.check_name_exists(space_id=space.space_id, model_name=model.model_name):
                    logger.warning(
                        f"System embedding model name {model.model_name} already exists in space: {space.space_id}")
                    continue
                model_dict.update({'space_id': space.space_id})
                model_config = self.model_repo.create(model_dict)
                logger.info(
                    f"Created embedding model config: {model_config.model_name}, id: {model_config.id} "
                    f"in space: {space.space_id}")

            return system_model
        except ModelConfigNameExistsError:
            raise
        except Exception as e:
            logger.error(f"Failed to create system embedding model: {str(e)}")
            self.db.rollback()
            raise ValidationError(f"Failed to create system embedding model: {str(e)}") from e

    def update_system_embedding_model(self, model: SystemEmbeddingModelUpdate) -> SystemEmbeddingModelDB:
        """Update system embedding model to the database.
        Args:
            model: SystemEmbeddingModelUpdate
        Returns:
            Updated system embedding model obj
        """
        try:
            # check system model exists
            if not self.system_model_repo.check_model_exists(model_id=model.id):
                raise ModelConfigNotFoundError("System embedding model id not exists")

            model_in_db = self.system_model_repo.get_by_id(model.id)

            # check model name conflict
            name_changed = model.model_name and model_in_db.model_name != model.model_name
            if name_changed:
                if self.system_model_repo.check_model_exists(name=model.model_name):
                    raise ModelConfigNameExistsError(f"System embedding model name already exists: {model.model_name}")

            model_dict = model.model_dump(exclude_unset=True, exclude_none=True)

            # Encrypt API key
            encrypted_api_key = self.security_utils.encrypt_api_key(model.api_key) if model.api_key else None
            if encrypted_api_key:
                model_dict.update({'api_key': encrypted_api_key})

            # update system model
            updated_model = self.system_model_repo.update(model_in_db, model_dict)
            logger.info(f"Created system embedding model name: {updated_model.model_name}, id: {updated_model.id}")

            # update this model for all users
            model_dict.update({'is_system_model': True})
            model_dict.update({'system_model_id': updated_model.id})

            all_spaces = self.space_repo.query().all()
            for space in all_spaces:
                if name_changed and self.model_repo.check_name_exists(space.space_id, model.model_name):
                    logger.warning(
                        f"System embedding model name {model.model_name} already exists in space: {space.space_id}")
                    continue

                model_config_in_db = self.model_repo.get_by_space_id_and_system_model_id(
                    space_id=space.space_id,
                    system_model_id=updated_model.id
                )
                if model_config_in_db:
                    updated_model_config = self.model_repo.update(model_config_in_db, model_dict)
                    logger.info(
                        f"Updated embedding model config: {updated_model_config.model_name}, "
                        f"id: {updated_model_config.id} in space: {updated_model_config.space_id}")
                else:
                    copy_dict = model_dict.copy()
                    copy_dict.update({'space_id': space.space_id})
                    model_config = self.model_repo.create(copy_dict)
                    logger.info(
                        f"Created embedding model config: {model_config.model_name}, id: {model_config.id} "
                        f"in space: {model_config.space_id}")

            return updated_model
        except (ModelConfigNotFoundError, ModelConfigNameExistsError):
            raise
        except Exception as e:
            logger.error(f"Failed to update system embedding model: {str(e)}")
            self.db.rollback()
            raise ValidationError(f"Failed to update system embedding model: {str(e)}") from e

    def get_system_embedding_model_with_pagination(
            self,
            page: int = 1,
            size: int = 10,
            filters: Optional[Dict[str, Any]] = None,
            sort_by: Optional[str] = 'update_time',
            sort_order: Optional[str] = 'desc'
    ) -> tuple[List[SystemEmbeddingModelDB], int]:
        """Get system embedding model list with pagination

        Args:
            page: Page number
            size: Page size
            filters: Filter conditions dictionary (Temporary not supported)
            sort_by: Sort by
            sort_order: Sort order (asc or desc)
        Returns:
            (System embedding model list, total count)
        """
        try:
            models, total = self.system_model_repo.get_paginated(
                page=page,
                size=size,
                sort_by=sort_by,
                sort_order=sort_order
            )

            logger.info(f"Paginated system embedding model query successful: page={page}, size={size}, total={total}")
            return models, total

        except Exception as e:
            logger.error(f"Paginated system embedding model query failed: {str(e)}")
            raise ValidationError(f"Paginated system llm model query failed: {str(e)}") from e



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.
from functools import wraps
from typing import Callable

from fastapi import status

from openjiuwen_studio.models.plugin import ToolBaseDB
from openjiuwen_studio.schemas.common import ResponseModel
from openjiuwen_studio.core.manager.repositories import JiuwenBaseRepository
from openjiuwen_studio.core.manager.repositories.jiuwen_base_repository import get_val_from_dict, get_db_jw
from openjiuwen.core.common.logging import logger
from openjiuwen_studio.core.database import milliseconds


class ToolRepository():
    def __init__(self) -> None:
        pass

    def with_exception_handling(func) -> Callable:
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            try:
                return func(self, *args, **kwargs)
            except Exception as e:
                logger.error(f"Error: tool db data preprocessing failed: {type(e).__name__}")
                return ResponseModel(code=status.HTTP_400_BAD_REQUEST,
                                     message=f"Error: tool db data preprocessing failed: {type(e).__name__}").model_dump(
                    exclude_none=True)

        return wrapper

    @with_exception_handling
    def tool_create(self, tool_data: dict) -> dict:
        with get_db_jw() as db:
            tool_db = JiuwenBaseRepository(db, ToolBaseDB)
            if not tool_data:
                logger.debug("No tool data to register")
                return ResponseModel(code=status.HTTP_400_BAD_REQUEST,
                                     message="No tool data to register").model_dump(exclude_none=True)
            find_id = {
                "tool_id": get_val_from_dict(tool_data, ["tool_id"]),
            }
            if not tool_data.get('plugin_version', ""):
                tool_data['plugin_version'] = ToolBaseDB.__version_none__
            timestamp = milliseconds()
            if "create_time" not in tool_data:
                tool_data["create_time"] = timestamp
            if "update_time" not in tool_data:
                tool_data["update_time"] = timestamp
            return tool_db.register_dl_in_sql(find_id=find_id, dl=tool_data).model_dump(exclude_none=True)

    @with_exception_handling
    def tool_get(self, query_body: dict) -> (dict, dict):
        with get_db_jw() as db:
            tool_db = JiuwenBaseRepository(db, ToolBaseDB)
            find_id = {	 
                "tool_id": get_val_from_dict(query_body, ["tool_id"]),	 
                "space_id": get_val_from_dict(query_body, ["space_id", "spaceId"]), 
            }
            res = tool_db.get_dl_in_sql(find_id=find_id, return_first_item=True, return_declarativebase=True)
            if res.code != status.HTTP_200_OK or not res.data:
                return res.model_dump(exclude_none=True), {}
            return res.model_dump(exclude_none=True), res.data.plugin_info.to_dict() if res.data.plugin_info is not None else {}

    @with_exception_handling
    def tool_save(self, tool_data: dict) -> dict:
        with get_db_jw() as db:
            tool_db = JiuwenBaseRepository(db, ToolBaseDB)
            if not tool_data:
                logger.debug("No tool data to update")
                return ResponseModel(code=status.HTTP_400_BAD_REQUEST,
                                     message="No tool data to update").model_dump(exclude_none=True)
            find_id = {
                "tool_id": get_val_from_dict(tool_data, ["tool_id"]),
                "space_id": get_val_from_dict(tool_data, ["space_id", "spaceId"]),
            }
            # 添加 plugin_version 到 find_id，因为唯一约束是 (tool_id, plugin_version)
            # 如果没有提供 plugin_version，使用默认值
            plugin_version = get_val_from_dict(tool_data, ["plugin_version"])
            if not plugin_version:
                plugin_version = ToolBaseDB.__version_none__
            find_id["plugin_version"] = plugin_version
            find_id = ToolBaseDB.filter_invalid_keys(find_id)
            timestamp = milliseconds()
            if "update_time" not in tool_data:
                tool_data["update_time"] = timestamp
            return tool_db.update_dl_in_sql(find_id=find_id, update_dl=tool_data).model_dump(exclude_none=True)

    @with_exception_handling
    def tool_delete(self, query_body: dict) -> dict:
        with get_db_jw() as db:
            tool_db = JiuwenBaseRepository(db, ToolBaseDB)
            # 删除操作需要谨慎，所以这里要求find_id必须所有值都非空
            tool_id = get_val_from_dict(query_body, ["tool_id"])
            space_id = get_val_from_dict(query_body, ["space_id", "spaceId"])
            if not tool_id or not space_id:
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST,
                    message="tool_data must contains tool_id and space_id/spaceId in order to delete"
                ).model_dump(exclude_none=True)
            find_id = {
                "tool_id": tool_id,
                "space_id": space_id,
            }
            return tool_db.unregister_dl_in_sql(find_id=find_id).model_dump(exclude_none=True)

    @with_exception_handling
    def tool_list(self, query_body: dict) -> dict:
        with get_db_jw() as db:
            tool_db = JiuwenBaseRepository(db, ToolBaseDB)
            find_id = {
                "space_id": get_val_from_dict(query_body, ["space_id", "spaceId"]),
            }
            res = tool_db.get_dl_in_sql(find_id=find_id)
            if res.code != status.HTTP_200_OK or not res.data:
                return res.model_dump(exclude_none=True)
            for idx, data in enumerate(res.data):
                if data.get("tool_version", None) == ToolBaseDB.__version_none__:
                    res.data[idx].pop("tool_version")
            return res.model_dump(exclude_none=True)

    @with_exception_handling
    def tool_update_available(self, tool_id: str, space_id: str, available: bool, plugin_version: str = None) -> dict:
        """
        更新工具的可用状态

        Args:
            tool_id: 工具ID
            space_id: 空间ID
            available: 工具是否可用
            plugin_version: 插件版本（可选，默认使用 __version_none__）

        Returns:
            dict: 更新结果
        """
        with get_db_jw() as db:
            tool_db = JiuwenBaseRepository(db, ToolBaseDB)

            if not tool_id or not space_id:
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST,
                    message="tool_id and space_id are required"
                ).model_dump(exclude_none=True)

            find_id = {
                "tool_id": tool_id,
                "space_id": space_id,
            }

            # 如果没有提供 plugin_version，使用默认值
            if not plugin_version:
                plugin_version = ToolBaseDB.__version_none__
            find_id["plugin_version"] = plugin_version
            find_id = ToolBaseDB.filter_invalid_keys(find_id)

            # 只更新 available 字段和 update_time
            update_data = {
                "available": available,
                "update_time": milliseconds()
            }

            return tool_db.update_dl_in_sql(find_id=find_id, update_dl=update_data).model_dump(exclude_none=True)


tool_repository = ToolRepository()


from fastapi import FastAPI, APIRouter, Request
from openjiuwen.core.common.logging import set_session_id, logger

from openjiuwen_studio.core.common.language_thread_context import (set_language, clear_language,
                                                                   get_highest_priority_language)
from openjiuwen_studio.core.config import settings
from openjiuwen_studio.routers import (auth, auth_new, models, users, agents, workflows, execution, space, deepsearch,
                                       related_member, plugin, tags, knowledge_base, embedding_models, prompt_router,
                                       prompt_debug_router, prompt_tuning_router, prompt_llm_router, system_model,
                                       memory_base)

api_router = APIRouter()


def router_register(app: FastAPI):
    """Register API routers to FastAPI app."""
    v1_router = APIRouter(prefix="/v1")
    if settings.enable_new_auth:
        # 新流程：挂载包含注册、密码管理的路由
        v1_router.include_router(auth_new.auth_router, prefix="/auth", tags=["New Auth"])
    else:
        # 挂载旧的直接登录逻辑
        v1_router.include_router(auth.auth_router, prefix="/auth", tags=["Legacy Auth"])
    v1_router.include_router(users.users_router, prefix="/users", tags=["Users"])
    v1_router.include_router(space.space_router, prefix="/spaces", tags=["Space"])
    v1_router.include_router(models.models_router, prefix="/models", tags=["Models"])
    v1_router.include_router(embedding_models.embedding_models_router, tags=["Embedding Models"])
    v1_router.include_router(agents.agents_router, prefix="/agents", tags=["Agents"])
    v1_router.include_router(execution.execution_router, prefix="/execution", tags=["Execution"])
    v1_router.include_router(workflows.workflows_router, prefix="/workflows", tags=["Workflows"])
    v1_router.include_router(deepsearch.deepsearch_router, prefix="/agent/deepsearch", tags=["deepsearch"])
    v1_router.include_router(related_member.related_router, prefix="/related", tags=["Relation"])
    v1_router.include_router(plugin.plugin_router, prefix="/plugin", tags=["Plugin"])
    v1_router.include_router(tags.tags_router, prefix="/tags", tags=["Tags"])
    v1_router.include_router(knowledge_base.knowledge_base_router, prefix="/knowledge-base", tags=["Knowledge Base"])
    v1_router.include_router(system_model.system_router, prefix="/system", tags=["System"])
    v1_router.include_router(memory_base.memory_base_router, prefix="/memory", tags=["Memory Base"])

    # Add health check endpoint directly to api_router (not v1_router)
    @api_router.get("/health")
    async def health_check():
        return {
            "status": "healthy",
            "service": "Jiuwen Agent Studio Backend",
            "version": "1.0.0"
        }

    api_router.include_router(v1_router)
    app.include_router(api_router, prefix="/api")

    app.include_router(prompt_llm_router.router)
    app.include_router(prompt_router.router)
    app.include_router(prompt_debug_router.router)
    app.include_router(prompt_tuning_router.router)

    @app.get("/")
    async def root():
        return {
            "message": "Welcome to Jiuwen Agent Studio Backend",
            "docs": "/api/docs",
            "health": "/api/health"
        }

    @app.middleware("http")
    async def process_header_language(request: Request, call_next):
        accept_language = request.headers.get("accept-language", "cn")
        accept_language_list = get_highest_priority_language(accept_language)
        if accept_language_list:
            language = accept_language_list[0]
        else:
            """ Default to 'cn' if no valid language is found in accept-language header """
            language = "cn"

        set_language(language)
        try:
            response = await call_next(request)
            return response
        finally:
            clear_language()

    @app.middleware("http")
    async def process_header_request_id(request: Request, call_next):
        request_id = request.headers.get("x-request-id")
        if request_id:
            set_session_id(request_id)
        try:
            response = await call_next(request)
            return response
        finally:
            logger.info("request %s, finish", request_id)



from .base_repository import BaseRepository
from .model_config_repository import ModelConfigRepository
from .model_usage_repository import ModelUsageRepository
from .embedding_model_config_repository import EmbeddingModelConfigRepository
from openjiuwen_studio.core.manager.repositories.jiuwen_base_repository import JiuwenBaseRepository
from .agent_repository import AgentRepository
from .prompt_relation_repository import PromptRelationRepository
from .user_repository import UserRepository
from .workflow_repository import WorkflowRepository
from .workflow_execution_repository import WorkflowExecutionRepository
from .agent_execution_repository import AgentExecutionRepository
from .awp_relation_repository import AgentWorkflowRelationDB

__all__ = [
    "BaseRepository",
    "ModelConfigRepository", 
    "ModelUsageRepository",
    "EmbeddingModelConfigRepository",
    "JiuwenBaseRepository",
    "AgentRepository",
    "PromptRelationRepository",
    "UserRepository",
    "WorkflowRepository",
    "WorkflowExecutionRepository",
    "AgentExecutionRepository",
    "AgentWorkflowRelationDB"
]


"""
Trace Detail Repository Implementation

Based on design document: WORKFLOW_TRACE_SCHEMA.md
Unified trace data repository, replacing the original workflow and agent separate design
"""

from functools import wraps
from typing import Any, Dict, List, Optional

from fastapi import status

from openjiuwen.core.common.logging import logger
from openjiuwen_studio.core.manager.repositories import JiuwenBaseRepository
from openjiuwen_studio.core.manager.repositories.jiuwen_base_repository import get_db_jw
from openjiuwen_studio.models.trace_detail import TraceDetailDB
from openjiuwen_studio.schemas.common import ResponseModel
from openjiuwen_studio.schemas.trace_detail import TraceDetail


def with_exception_handling(func):
    """Exception handling decorator"""

    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            logger.error(f"Trace detail repository error: {str(e)}")
            return ResponseModel(
                code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                message=f"Operation failed: {str(e)}",
                data=None,
            )

    return wrapper


class TraceDetailRepository:
    """Unified Trace Detail Repository"""

    def __init__(self):
        pass

    @with_exception_handling
    def create_trace_detail(self, trace_data: TraceDetail) -> ResponseModel:
        """Create trace detail record

        Args:
            trace_data: TraceDetail model data

        Returns:
            ResponseModel: Creation result
        """
        with get_db_jw() as db:
            base_repo = JiuwenBaseRepository(db, TraceDetailDB)

            # Convert Pydantic model to dictionary
            trace_dict = trace_data.model_dump(exclude_unset=True)
            return base_repo.register_dl_in_sql(find_id=None, dl=trace_dict)

    @with_exception_handling
    def create_trace_details(self, trace_data_list: List[TraceDetail]) -> ResponseModel:
        """Batch create trace detail records for performance optimization

        Args:
            trace_data_list: List of TraceDetail model data

        Returns:
            ResponseModel: Batch creation result
        """
        if not trace_data_list:
            return ResponseModel(
                code=status.HTTP_200_OK,
                message="No trace details to save",
                data=None
            )
        
        with get_db_jw() as db:
            base_repo = JiuwenBaseRepository(db, TraceDetailDB)

            # Convert Pydantic models to dictionaries
            trace_dicts = [trace_data.model_dump() for trace_data in trace_data_list]
            return base_repo.bulk_register_dl(trace_dicts)

    @with_exception_handling
    def get_trace_details_by_trace_id(self, trace_id: str) -> ResponseModel:
        """Get trace detail record list by trace_id

        Args:
            trace_id: Trace ID

        Returns:
            ResponseModel: Trace record list
        """
        with get_db_jw() as db:
            base_repo = JiuwenBaseRepository(db, TraceDetailDB)

            find_id = {"trace_id": trace_id}
            result = base_repo.get_dl_in_sql_with_cols(
                find_id=find_id,
                order_cols_asc=["start_time_micros"]
            )

            if result.code != status.HTTP_200_OK:
                # Query failed, return failure status code
                return result

            if not result.data:
                return ResponseModel(
                    code=status.HTTP_404_NOT_FOUND,
                    message="Trace records not found",
                    data=[]
                )

            return ResponseModel(
                code=status.HTTP_200_OK,
                message=f"Retrieved {len(result.data)} trace records",
                data=result.data
            )

    @with_exception_handling
    def delete_trace_detail_by_trace_id(self, trace_id: str) -> ResponseModel:
        """Delete all related records by trace_id

        Args:
            trace_id: Trace ID

        Returns:
            ResponseModel: Deletion result
        """
        with get_db_jw() as db:
            base_repo = JiuwenBaseRepository(db, TraceDetailDB)

            find_id = {"trace_id": trace_id}

            # First query records to be deleted
            query_result = base_repo.get_dl_in_sql_with_cols(
                find_id=find_id
            )

            if query_result.code != status.HTTP_200_OK:
                return ResponseModel(
                    code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                    message="Failed to query records for deletion",
                    data=None
                )

            if not query_result.data:
                return ResponseModel(
                    code=status.HTTP_404_NOT_FOUND,
                    message="No records found to delete",
                    data=0
                )

            # Delete all matching records
            delete_result = base_repo.unregister_dl_in_sql(find_id)

            # Return deleted record count
            deleted_count = len(query_result.data) if query_result.data else 0
            return ResponseModel(
                code=delete_result.code,
                message=delete_result.message,
                data=deleted_count
            )


trace_detail_repository = TraceDetailRepository()


"""Model Manager - Public Interface Modules

This package contains the public interface classes for model management.
These are the main entry points that external code should use.
"""

from .embedding_model_config_manager import EmbeddingModelConfigManager
from .embedding_model_test_manager import EmbeddingModelTester
from .model_config_manager import ModelConfigManager
from .model_test_manager import ModelTester

from .system_embedding_model_manager import SystemEmbeddingModelManager

from .system_llm_model_manager import SystemLLMModelManager

__all__ = [
    "ModelConfigManager",
    "ModelTester",
    "EmbeddingModelConfigManager",
    "EmbeddingModelTester",
    "SystemLLMModelManager",
    "SystemEmbeddingModelManager",
]



import sys
from pathlib import Path
sys.path.append(str(Path(__file__).resolve().parent.parent.parent.parent))
from sqlalchemy import create_engine, text
from openjiuwen.core.common.logging import logger
from openjiuwen_studio.models.db_fun_base import Base
from openjiuwen_studio.models.agent import AgentBaseDB, AgentPublishDB, AgentBaseDBPd, AgentPublishDBPd
from openjiuwen_studio.models.user import UserDB, SpaceDB, SpaceUserDB
from openjiuwen_studio.models.workflow import WorkflowBaseDB, WorkflowPublishDB, WorkflowBaseDBPd, WorkflowPublishDBPd
from sqlalchemy.orm import sessionmaker

# 替换为你的MySQL数据库信息
USERNAME = 'root'
PASSWORD = ''
HOST = 'localhost'  # 例如：'localhost' 或 '127.0.0.1'
PORT = '33306'  # 通常是 3306
DATABASE = 'testdb'

# 创建连接引擎
# engine = create_engine(f'mysql+pymysql://{USERNAME}:{PASSWORD}@{HOST}:{PORT}/{DATABASE}')

engine = create_engine(f'mysql+mysqldb://{USERNAME}:{PASSWORD}@{HOST}:{PORT}/{DATABASE}',
                        pool_size=10,       # 连接池大小
                        max_overflow=20,    # 超过连接池大小外最多创建的连接
                        pool_timeout=30,
                        echo=True           # 可选：打印 SQL 日志
                        )
Base.metadata.create_all(engine)

Session = sessionmaker(bind=engine,
                       autoflush=True,
                       autocommit=False,
                       expire_on_commit=True)
session = Session()


def get_db():
    db = Session()
    try:
        yield db
    finally:
        db.close()


# -------------------- 快速测试 --------------------
if __name__ == "__main__":
    agent_dict = {"space_id": "1234", "agent_id": "agent-001",
        "agent_version": "draft", "other": "others", "create_time": 123}
    agent_db = AgentBaseDB.from_dict(agent_dict)
    session.query(AgentBaseDB).filter_by(agent_id="agent-001").delete()
    session.add(agent_db)
    session.commit()

    logger.info(agent_db)
    agent_publish_dict = {"space_id": "1234", "agent_id": "agent-001", "agent_version": "v1.0.0", "other": "others"}
    agent_publish_db = AgentPublishDB.from_dict(agent_publish_dict)
    session.query(AgentPublishDB).filter_by(agent_id="agent-001").delete()
    session.add(agent_publish_db)
    session.commit()
    logger.info(agent_publish_db)

    workflow_dict = {"space_id": "1234", "workflow_id": "wf-001", "workflow_version": "draft", "name": "test_name", 
                     "key1": "value1", "key2": "value2",
                     "schema": "{\"node\":dfkajfkjdkfhajdfhda jhfahdfjahdjfhlahkdfjhajdhfjahfjahdfjadlkfjkldhafjadklfdhfjlkhfaldfhafjhfhadjafhjdhfjhf}"}
    workflow_db = WorkflowBaseDB.from_dict(workflow_dict)
    session.query(WorkflowBaseDB).filter_by(workflow_id="wf-001").delete()
    session.add(workflow_db)
    session.commit()
    logger.info(workflow_db)
    workflow_publish_dict = {"space_id": "1234", "workflow_id": "wf-001", "workflow_version": "v1.0.0", "desc": "测试一下"}
    workflow_publish_db = WorkflowPublishDB.from_dict(workflow_publish_dict)
    session.query(WorkflowPublishDB).filter_by(workflow_id="wf-001").delete()
    session.add(workflow_publish_db)
    session.commit()
    
    user_dict = {"user_id_str": "test_user", "others": {"key1": "value1", "key2": "value2"}}
    user_db = UserDB.from_dict(user_dict)
    session.query(UserDB).filter_by(user_id_str="test_user").delete()
    session.add(user_db)
    session.commit()
    logger.info(user_db)

    space_dict = {"space_id": "1234", "user_id_str": "test", "description": "this is a test space"}
    space_db = SpaceDB.from_dict(space_dict)
    session.query(SpaceDB).filter_by(space_id="1234").delete()
    session.add(space_db)
    session.commit()
    logger.info(space_db)

    space_user_dict = {"space_id": "1234", "user_id_str": "test"}
    space_user_db = SpaceUserDB.from_dict(space_user_dict)
    session.query(SpaceUserDB).filter_by(space_id="1234").delete()
    session.add(space_user_db)
    session.commit()
    logger.info(space_user_db)

    a = 0




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.

from typing import Any, Dict
from openjiuwen.core.workflow import LLMCompConfig, LLMComponent
from openjiuwen.core.foundation.llm import ModelRequestConfig, ModelClientConfig, SystemMessage, UserMessage

from openjiuwen_studio.core.common.dsl import LLMConfig as LLMConfigDL
from openjiuwen_studio.core.executor.component.compile.base_comp_compiler import BaseCompCompiler

client_provider_mapping = {
    'siliconflow': "SiliconFlow",
    'openai': "OpenAI",
}


def parse_model_config(comp_config_dict: Dict[str, Any]) -> tuple[ModelRequestConfig, ModelClientConfig, str]:
    """
    解析模型配置，返回 ModelRequestConfig、ModelClientConfig 和 model_id
    Args:
        comp_config_dict: LLM组件配置字典
    Returns:
        tuple: (model_request_config, model_client_config, model_id)
    """
    llm_config_dl = LLMConfigDL.model_validate(comp_config_dict)
    base_model_request_config = llm_config_dl.model.request_config
    base_model_client_config = llm_config_dl.model.model_client_config

    # 创建 ModelRequestConfig
    model_request_config = ModelRequestConfig(
        model=base_model_request_config.model_name,
        temperature=base_model_request_config.temperature,
        top_p=base_model_request_config.top_p,
    )

    # 创建 ModelClientConfig
    model_client_config = ModelClientConfig(
        client_provider=client_provider_mapping[base_model_client_config.client_provider],
        api_key=base_model_client_config.api_key,
        api_base=base_model_client_config.api_base,
        timeout=base_model_client_config.timeout,
        max_retries=1,
        verify_ssl=False,
    )

    # model_id 使用 model_name
    model_id = base_model_request_config.model_name
    return model_request_config, model_client_config, model_id


def parse_template_content(template_content: list) -> tuple[SystemMessage, UserMessage]:
    """
    从 template_content 中解析出 system_prompt_template 和 user_prompt_template
    Args:
        template_content: 模板内容列表，每个元素是 {"role": "system"/"user", "content": "..."}
    Returns:
        tuple: (system_prompt_template, user_prompt_template)
    """
    system_prompt_template = None
    user_prompt_template = None
    if not template_content:
        return None, None

    for msg in template_content:
        if not isinstance(msg, dict):
            continue
        role = msg.get("role", "")
        content = msg.get("content", "")
        if role == "system":
            system_prompt_template = SystemMessage(content=content)
        elif role == "user":
            user_prompt_template = UserMessage(content=content)
    return system_prompt_template, user_prompt_template


class LLMCompCompiler(BaseCompCompiler):
    def __init__(self, llm_comp_config_dict: Dict[str, Any]) -> None:
        super().__init__()
        self.llm_comp_config_dict = llm_comp_config_dict
        self.llm_config_dl = LLMConfigDL.model_validate(llm_comp_config_dict)

    def compile(self) -> LLMComponent:
        model_request_config, model_client_config, model_id = parse_model_config(self.llm_comp_config_dict)
        # 解析 template_content 模板内容
        system_prompt_template, user_prompt_template = parse_template_content(self.llm_config_dl.template_content)

        # 创建 LLMCompConfig
        llm_comp_config = LLMCompConfig(
            model_id=None,  # 不能给，给了会走到Runner.get_model
            model_config=model_request_config,
            model_client_config=model_client_config,
            system_prompt_template=system_prompt_template,
            user_prompt_template=user_prompt_template,
            template_content=self.llm_config_dl.template_content,  # 下一个core版本要删除
            response_format={
                "type": self.llm_config_dl.response_format_type,
            },
            output_config=self.llm_config_dl.output_config,
            enable_history=self.llm_config_dl.enable_history,
            cache_stream=True
        )

        return LLMComponent(llm_comp_config)



import copy
from functools import wraps
from typing import Callable

from fastapi import status
from sqlalchemy.orm import Session

from openjiuwen.core.common.logging import logger
from openjiuwen_studio.core.database import milliseconds
from openjiuwen_studio.core.manager.repositories import JiuwenBaseRepository
from openjiuwen_studio.core.manager.repositories.agent_repository import agent_repository
from openjiuwen_studio.core.manager.repositories.jiuwen_base_repository import get_db_jw
from openjiuwen_studio.core.manager.repositories.workflow_repository import \
    workflow_repository
from openjiuwen_studio.models.awp_relation import AgentWorkflowRelationDB
from openjiuwen_studio.schemas.agent import AgentId
from openjiuwen_studio.schemas.common import ResponseModel
from openjiuwen_studio.schemas.workflow import WorkflowId


class AwpRelationRepository():
    def __init__(self) -> None:
        pass
    # def __init__(self, db: Session) -> None:
    #     # 关键：显式写出 [A] 和 [B]
    #     self._prompt_relation_db: JiuwenBaseRepository[PromptRelationDB] = JiuwenBaseRepository(db, PromptRelationDB)

    def with_exception_handling(func) -> Callable:
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            try:
                return func(self, *args, **kwargs)
            except Exception as e:
                logger.error("Data preprocessing error for relation of agent/workflow/plugin")
                logger.debug(f"Relationship processing exception: {type(e).__name__}", exc_info=True)
                return ResponseModel(code=status.HTTP_400_BAD_REQUEST, 
                                     message=f"Error: Data preprocessing error for relation of agent/workflow/plugin. \
                                                Exception details: {type(e).__name__}")
        return wrapper

    '''
    description: 为某个agent增加与workflow的关联
    param {AgentId} agent_id
    param {WorkflowId} workflow_id
    param {Session} db_session
    return {*}
    '''
    @with_exception_handling
    def create_agent_workflow_relation(self, agent_id: AgentId, workflow_id: WorkflowId, 
                                       db_session: Session | None = None) -> ResponseModel[None]:
        if agent_id.space_id != workflow_id.space_id:
            raise ValueError(f"agent_id and workflow_id must be in the same space, but got agent_id.space_id({agent_id.space_id}) \
                             != workflow_id.space_id({workflow_id.space_id})")
        with get_db_jw(db_session) as db:
            agent_id = agent_repository.patch_agent_id(agent_id, db)
            workflow_id = workflow_repository.patch_workflow_id(workflow_id, db)
            timestamp = milliseconds()
            aw_relation = AgentWorkflowRelationDB(
                space_id=agent_id.space_id,
                agent_id=agent_id.agent_id,
                agent_version=agent_id.agent_version,
                workflow_id=workflow_id.workflow_id,
                workflow_version=workflow_id.workflow_version,
                create_time=timestamp,
                update_time=timestamp,
            )
            aw_relation_db = JiuwenBaseRepository(db, AgentWorkflowRelationDB)
            return aw_relation_db.register_dl_in_sql(None, aw_relation)

    '''
    description: 删除某个agent-workflow的关联
    param {AgentId} agent_id
    param {WorkflowId} workflow_id
    param {Session} db_session
    return {*}
    '''
    @with_exception_handling
    def delete_agent_workflow_relation(self, agent_id: AgentId, workflow_id: WorkflowId, 
                                       db_session: Session | None = None) -> ResponseModel[None]:
        if agent_id.space_id != workflow_id.space_id:
            raise ValueError(f"agent_id and workflow_id must be in the same space, but got agent_id.space_id({agent_id.space_id}) \
                             != workflow_id.space_id({workflow_id.space_id})")
        with get_db_jw(db_session) as db:
            agent_id = agent_repository.patch_agent_id(agent_id, db)
            workflow_id = workflow_repository.patch_workflow_id(workflow_id, db)
            find_id = {
                "space_id": agent_id.space_id,
                "agent_id": agent_id.agent_id,
                "agent_version": agent_id.agent_version,
                "workflow_id": workflow_id.workflow_id,
                "workflow_version": workflow_id.workflow_version,
            }
            aw_relation_db = JiuwenBaseRepository(db, AgentWorkflowRelationDB)
            return aw_relation_db.unregister_dl_in_sql(find_id=find_id)
    
    '''
    description: 复制关联项: 需要发布agent或者复制agent, 将原版本agent的所有关联项复制后关联到新版本的agent中。
    param {AgentId} agent_id_source     原版本agent
    param {AgentId} agent_id_target     目标版本agent
    param {Session} db_session
    return {*}
    '''
    @with_exception_handling
    def copy_agent_workflow_relation(self, agent_id_source: AgentId, agent_id_target: AgentId, 
                                     db_session: Session | None = None) -> ResponseModel[None]:
        if agent_id_source.space_id != agent_id_target.space_id:
            raise ValueError(f"agent_id_source and agent_id_target must be in the same space, but got agent_id_source.space_id({agent_id_source.space_id}) \
                             != agent_id_target.space_id({agent_id_target.space_id})")
        with get_db_jw(db_session) as db:
            agent_id_source = agent_repository.patch_agent_id(agent_id_source, db)
            agent_id_target = agent_repository.patch_agent_id(agent_id_target, db)
            # 判断目标agent与源版本是否完全相同，完全相同则抛出异常
            if agent_id_source.agent_id == agent_id_target.agent_id and agent_id_source.agent_version == agent_id_target.agent_version:
                raise ValueError(f"agent_id_source and agent_id_target must be different, but got same agent_id({agent_id_source.agent_id}) \
                             and same agent_version({agent_id_source.agent_version}).")
            aw_relation_db = JiuwenBaseRepository(db, AgentWorkflowRelationDB)
            # 获取agent-workflow的所有关联数据
            db_res = aw_relation_db.get_dl_in_sql(find_id=agent_id_source.model_dump())
            if db_res.code != status.HTTP_200_OK:
                return db_res
            aw_relation_list: list[dict] = db_res.data
            # 更新关联数据
            for relation in aw_relation_list:
                relation.pop('primary_id', None)
                relation['agent_id'] = agent_id_target.agent_id
                relation['agent_version'] = agent_id_target.agent_version
            return aw_relation_db.bulk_register_dl(aw_relation_list)
    
    '''
    description: 发布agent, 根据发布的agent_id来复制相应的aw关系到新版本的agent中。
    param {AgentId} agent_id_publish    待发布的agent_id
    param {Session} db_session          数据库会话(可选)
    return {*}
    '''
    @with_exception_handling
    def copy_agent_workflow_relation_for_publish(self, agent_id_publish: AgentId, 
                                                 db_session: Session | None = None) -> ResponseModel[None]:
        agent_id_draft = copy.deepcopy(agent_id_publish)
        agent_id_draft.agent_version = None
        return self.copy_agent_workflow_relation(agent_id_draft, agent_id_publish, db_session)

    '''
    description: 查询关联项: 根据输入的agent/workflow id, 查询所有的agent/workflow关联项
    param {*} self
    param {AgentId} query_id
    param {Session} db_session
    return {*}
    '''
    @with_exception_handling
    def get_agent_workflow_relation(self, query_id: AgentId | WorkflowId, 
                                    db_session: Session | None = None) -> ResponseModel[list[dict] | None]:
        with get_db_jw(db_session) as db:
            if isinstance(query_id, AgentId):
                query_id = agent_repository.patch_agent_id(query_id, db)
            else:
                query_id = workflow_repository.patch_workflow_id(query_id, db)
            aw_relation_db = JiuwenBaseRepository(db, AgentWorkflowRelationDB)
            return aw_relation_db.get_dl_in_sql(find_id=query_id.model_dump())


awp_relation_repository = AwpRelationRepository()


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.

import asyncio
import threading
from dataclasses import dataclass
from typing import Dict, Optional
from openjiuwen.core.common.logging import logger
from openjiuwen.core.workflow.components import Session


@dataclass
class ComponentExecutionRegistration:
    execution_id: str
    session: Optional[Session] = None
    task: Optional[asyncio.Task] = None


@dataclass
class ComponentExecutionInfo(ComponentExecutionRegistration):
    thread_id: Optional[int] = None


class ComponentExecutionManager:
    def __init__(self):
        self._executions: Dict[str, ComponentExecutionInfo] = {}
        self._cancelled_flags: Dict[str, bool] = {}
        self._lock = threading.Lock()

    def register_execution(self, reg: ComponentExecutionRegistration):
        with self._lock:
            key = reg.execution_id
            self._executions[key] = ComponentExecutionInfo(
                execution_id=key, session=reg.session, task=reg.task, thread_id=threading.get_ident()
            )
            self._cancelled_flags[key] = False
            logger.info(f"Registered component execution: {key}")

    def unregister_execution(self, execution_id: str):
        with self._lock:
            if execution_id in self._executions:
                self._executions.pop(execution_id)
                self._cancelled_flags.pop(execution_id, None)
                logger.info(f"Unregistered component execution: {execution_id}")

    def is_cancelled(self, execution_id: str) -> bool:
        with self._lock:
            return self._cancelled_flags.get(execution_id, False)

    def get_execution(self, execution_id: str) -> Optional[ComponentExecutionInfo]:
        with self._lock:
            return self._executions.get(execution_id)

    async def cancel_execution(self, execution_id: str) -> bool:
        info = self.get_execution(execution_id)
        if not info:
            logger.warning(f"Component execution not found: {execution_id}")
            return False
        try:
            with self._lock:
                self._cancelled_flags[execution_id] = True
            logger.info(f"Set cancelled flag for {execution_id}")

            if info.task and not info.task.done():
                info.task.cancel()
                logger.info(f"Component run task cancelled by user: {execution_id}")
                try:
                    await info.task
                except asyncio.CancelledError:
                    logger.info(f"Task cancelled: {execution_id}")
                except Exception as e:
                    logger.error(f"Cancel task error: {e}", exc_info=True)

            self.unregister_execution(execution_id)
            return True
        except Exception as e:
            logger.error(f"Cancel component execution error: {e}", exc_info=True)
            return False


component_execution_manager = ComponentExecutionManager()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.
from enum import Enum

from openjiuwen_studio.core.common import dsl
from openjiuwen_studio.core.common.dsl import LoopType
from openjiuwen_studio.schemas.node import Node, Inputs
from openjiuwen_studio.core.manager.convertor.components.common import outputs_convert, input_params_convert, \
    base_value_convert
from openjiuwen_studio.core.common.dsl import ComponentType

MAX_LOOP_NUM = 1000


def _loop_inputs_convert(inputs: Inputs) -> dsl.LoopInput:
    loop_inputs = dsl.LoopInput()
    loop_param = inputs.loop_param
    if loop_param is None:
        raise TypeError("loopParam is none")

    loop_inputs.loop_type = loop_param.type
    if loop_param.type == "numLoop":
        loop_inputs.loop_type = LoopType.Number.value
        loop_inputs.loop_number = base_value_convert(loop_param.loop_num)
        loop_num = loop_inputs.loop_number
        if isinstance(loop_num, int) and (loop_num < 1 or loop_num > MAX_LOOP_NUM):
            raise ValueError(f"loop num must be between 1 and {MAX_LOOP_NUM}")
    elif loop_param.type == "arrayLoop":
        loop_inputs.loop_type = LoopType.Array.value
        loop_array = loop_param.loop_array
        if loop_array is None:
            raise TypeError("loop array is none")
        loop_inputs.loop_array = input_params_convert(loop_array)
    else:
        raise ValueError("unknow loop type")

    intermediate_var = loop_param.intermediate_var
    if intermediate_var is not None:
        loop_inputs.intermediate_var = input_params_convert(intermediate_var)

    return loop_inputs


def loop_convert(node: Node) -> dsl.Component:
    try:
        data = node.data
        outputs = data.outputs
        if outputs is None:
            raise TypeError("outputs is none")
        convert_output = outputs_convert(outputs)

        inputs = data.inputs
        if inputs is None:
            raise TypeError("inputs is none")
        convert_inputs = _loop_inputs_convert(inputs)

        return dsl.Component(
            id=node.id,
            type=ComponentType.COMPONENT_TYPE_LOOP,
            type_version="1.0.0",
            inputs=convert_inputs.model_dump(),
            outputs=convert_output,
            configs={},
            description="",
            name=data.title
        )
    except Exception as e:
        raise ValueError(f"Failed to convert loop node: {str(e)}") from e


def loop_continue_convert(node: Node) -> dsl.Component:
    try:
        data = node.data

        return dsl.Component(
            id=node.id,
            type=ComponentType.COMPONENT_TYPE_CONTINUE,
            type_version="1.0.0",
            configs={},
            description="",
            name=data.title
        )
    except Exception as e:
        raise ValueError(f"Failed to convert loop continue node: {str(e)}") from e


def loop_break_convert(node: Node) -> dsl.Component:
    try:
        data = node.data

        return dsl.Component(
            id=node.id,
            type=ComponentType.COMPONENT_TYPE_BREAK,
            type_version="1.0.0",
            configs={},
            description="",
            name=data.title
        )
    except Exception as e:
        raise ValueError(f"Failed to convert loop break node: {str(e)}") from e



"""fix sqlite agent update

Revision ID: 8f4846812221
Revises: 07acd03b1173
Create Date: 2026-02-07 11:50:21.774021

"""
from typing import Sequence, Union

from alembic import op
from openjiuwen_studio.core.database.migration_utils import table_exists, column_exists, index_exists
import sqlalchemy as sa
from sqlalchemy import inspect


# revision identifiers, used by Alembic.
revision: str = '8f4846812221'
down_revision: Union[str, Sequence[str], None] = '07acd03b1173'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    if not table_exists('memory_base'):
        op.create_table('memory_base',
        sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
        sa.Column('space_id', sa.String(length=100), nullable=False,
                     comment='空间ID，用于多租户隔离'),
        sa.Column('mdb_id', sa.String(length=100), nullable=False, comment='记忆库ID，唯一标识'),
        sa.Column('name', sa.String(length=100), nullable=False, comment='记忆库名称'),
        sa.Column('description', sa.String(length=100), nullable=False, comment='记忆库名描述'),
        sa.Column('embedding_model_config_id', sa.Integer(), nullable=False,
                     comment='Embedding 模型配置ID'),
        sa.Column('llm_model_config_id', sa.Integer(), nullable=False,
                     comment='LLM 模型配置ID'),
        sa.Column('create_time', sa.BigInteger(), nullable=False, comment='创建时间'),
        sa.Column('update_time', sa.BigInteger(), nullable=False, comment='更新时间'),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('mdb_id', name='uix_mdb_id'),
        comment='记忆库表，存储记忆库基本信息'
        )

    with op.batch_alter_table('memory_base', schema=None) as batch_op:
        if not index_exists('memory_base', 'idx_space_id_memory_base'):
            batch_op.create_index('idx_space_id_memory_base', ['space_id'], unique=False)
        if not index_exists('memory_base', 'idx_space_mdb_memory_base'):
            batch_op.create_index('idx_space_mdb_memory_base', ['space_id', 'mdb_id'], unique=False)
        if not index_exists('memory_base', 'ix_memory_base_embedding_model_config_id_memory_base'):
            batch_op.create_index('ix_memory_base_embedding_model_config_id_memory_base',
                                  ['embedding_model_config_id'], unique=False)
        if not index_exists('memory_base', 'ix_memory_base_llm_model_config_id_memory_base'):
            batch_op.create_index('ix_memory_base_llm_model_config_id_memory_base',
                                  ['llm_model_config_id'], unique=False)
        if not index_exists('memory_base', 'ix_memory_base_mdb_id_memory_base'):
            batch_op.create_index('ix_memory_base_mdb_id_memory_base', ['mdb_id'], unique=False)

    if table_exists('knowledge_base'):
        with op.batch_alter_table('knowledge_base', schema=None) as batch_op:
            if not column_exists('knowledge_base', 'index_manager_type'):
                batch_op.add_column(sa.Column('index_manager_type',
                                                 sa.String(length=200), nullable=True,
                                                 comment='Whether Chroma or Milvus is used for indexing'))

    if table_exists('knowledge_base_document'):
        with op.batch_alter_table('knowledge_base_document', schema=None) as batch_op:
            if not column_exists('knowledge_base_document', 'index_manager_type'):
                batch_op.add_column(sa.Column('index_manager_type',
                                                 sa.String(length=200), nullable=True,
                                                 comment='Whether Chroma or Milvus is used for indexing'))
            if not column_exists('knowledge_base_document', 'index_id'):
                batch_op.add_column(sa.Column('index_id', sa.String(length=200),
                                                 nullable=True, comment='ES中的索引ID'))
            if not column_exists('knowledge_base_document', 'index_name'):
                batch_op.add_column(sa.Column('index_name', sa.String(length=200),
                                                 nullable=True, comment='ES索引名称'))
            if column_exists('knowledge_base_document', 'es_index_name'):
                batch_op.drop_column('es_index_name')
            if column_exists('knowledge_base_document', 'es_index_id'):
                batch_op.drop_column('es_index_id')

    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    if table_exists('knowledge_base_document'):
        with op.batch_alter_table('knowledge_base_document', schema=None) as batch_op:
            if not column_exists('knowledge_base_document', 'es_index_id'):
                batch_op.add_column(sa.Column('es_index_id', sa.VARCHAR(length=200), nullable=True))
            if not column_exists('knowledge_base_document', 'es_index_name'):
                batch_op.add_column(sa.Column('es_index_name', sa.VARCHAR(length=200), nullable=True))
            if column_exists('knowledge_base_document', 'index_name'):
                batch_op.drop_column('index_name')
            if column_exists('knowledge_base_document', 'index_id'):
                batch_op.drop_column('index_id')
            if column_exists('knowledge_base_document', 'index_manager_type'):
                batch_op.drop_column('index_manager_type')

    if table_exists('knowledge_base'):
        with op.batch_alter_table('knowledge_base', schema=None) as batch_op:
            if column_exists('knowledge_base', 'index_manager_type'):
                batch_op.drop_column('index_manager_type')

    if table_exists('memory_base'):
        with op.batch_alter_table('memory_base', schema=None) as batch_op:
            if index_exists('memory_base', 'ix_memory_base_mdb_id_memory_base'):
                batch_op.drop_index('ix_memory_base_mdb_id_memory_base')
            if index_exists('memory_base', 'ix_memory_base_llm_model_config_id_memory_base'):
                batch_op.drop_index('ix_memory_base_llm_model_config_id_memory_base')
            if index_exists('memory_base', 'ix_memory_base_embedding_model_config_id_memory_base'):
                batch_op.drop_index('ix_memory_base_embedding_model_config_id_memory_base')
            if index_exists('memory_base', 'idx_space_mdb_memory_base'):
                batch_op.drop_index('idx_space_mdb_memory_base')
            if index_exists('memory_base', 'idx_space_id_memory_base'):
                batch_op.drop_index('idx_space_id_memory_base')

    if table_exists('memory_base'):
        op.drop_table('memory_base')
    # ### end Alembic commands ###